#!/usr/bin/perl -w
###############################################################################
#    Copyright (C) 2002-2204 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id: afickonfig.pl,v 1.4 2004/06/23 15:17:28 gerbier Exp $
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
#                         perl modules
###############################################################################

use strict;

use Getopt::Long;
use Carp qw(cluck);

###############################################################################
#                     global variables
###############################################################################

my $Version = '0.1-0';

# connection between command line options and config parameters

#######################################################
# usage
# print some help
sub usage($) {
	my $version = shift (@_);
	print <<EOHELP;

Usage: $0 [afickonfig option] [afick options]

afickonfig options  : 
 -c|--config_file file        configname of config file to use
 -h|--help                    show this help page
 --print_config               display internals variables after arguments and config file parsing
 				(for debugging purposes)
 -V|--version                 show afickonfig version

afick options :

 -a|--ignore_case             helpful on windows plateforms, dangerous on unix ones
 				reverse : --noignore_case
 -d|--debug level	      set a level of debugging messages, from 0 (none) to 3 (full)
 				default : 0
 -D| --database file          force the database name 
 -f|--full_newdel             report full information for new or deleted directories
                               default: no
			       reverse : --nofull_newdel 
 -m|--missing_files           warn about files declared in config files 
                               which do not exists, 
			       default: no; 
			       reverse : --nomissing_files
 -r|--running_files           warn about "running" files : modified since program begin
                               default: no 
                               reverse: --norunning_files
 -s|--dead_symlinks           warn about dead symlinks 
                               default: no 
                               reverse: --nodead_symlinks
 -t|--timing		      Print timing statistics
				default: no
				reverse : --notiming
 -x|--exclude_suffix ext1 ext2        list of suffixes to ignore
 -y|--history file	      history file of all runs with summary
 -archive directory	      directory where archive files are stored

Disclaimer:
This script is intended to provide a means for
detecting changes made to files, via a regular
comparison of MD5 hashes to an established baseline. 

Copyright (c) 2002 Eric Gerbier <gerbier\@users.sourceforge.net>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
EOHELP

}

#############################################################
sub print_config ($) {
	my $configfile = shift (@_);
	system("afick -c $configfile --print_config");
}
#############################################################
sub change_config($%) {
	my $configfile = shift (@_);
	my %changes    = @_;

	open( CONFIG, '<', $configfile ) or die "can not open $configfile : $!\n";
	my @config = <CONFIG>;
	close(CONFIG);

	my $nb_changes = 0;
	foreach my $key ( keys %changes ) {
		my $found     = 0;
		my @newconfig = ();
		foreach my $ligne (@config) {
			if ( $ligne =~ m/^\s*$key\s*:=\s*(.*)\s*$/ ) {
				my $val = $1;
				if ( $val eq $changes{$key} ) {
					print "no changes for $key\n";
				}
				else {
					print "replace $ligne by $key := $changes{$key}\n";
					$ligne = "$key := $changes{$key}\n";
					$nb_changes++;
				}
				$found = 1;
			}
			push ( @newconfig, $ligne );
		}
		if ( !$found ) {
			print "add line $key := $changes{$key}\n";
			push ( @newconfig, "$key := $changes{$key}\n" );
			$nb_changes++;
		}
		@config = @newconfig;
	}
	if ($nb_changes) {

		# save modified config file
		print "rewrite changed $configfile ($nb_changes)\n";
		open( CONFIG, '>', $configfile )
		  or die "can not write to $configfile : $!\n";
		foreach my $ligne (@config) {
			print CONFIG $ligne;
		}
		close(CONFIG);
	}
	return $nb_changes;
}
#############################################################
sub version($) {
	my $version = shift (@_);
	print "\n";
	print
"afickonfig : another file integrity checker configurator\nversion $version\n";
}
#############################################################
#                          main
#############################################################

my $default_config_file;
if ( $^O =~ m/linux/ ) {
	$default_config_file = '/etc/afick.conf';
}
elsif ( $^O =~ m/^MSWin/ ) {
	$default_config_file = 'xp.conf';

}
else {
	$default_config_file = 'afick.conf';
}

$| = 1;

# variables for parameter analysis
my $configfile;    # config file name
my $help;
my $version;
my $print_config;

# parameters for afick
my (
	$Archive,   $Debug_level,       $Ignore_case, $Report_full_newdel,
	$History,   $Warn_missing_file, $Running,     $Warn_dead_symlinks,
	$sufx_list, $Timing,            $Database
);

Getopt::Long::Configure('no_ignore_case');
unless (
	GetOptions(

		# afickonfig options
		'config_file|c=s' => \$configfile,
		'help|h'          => \$help,
		'print_config'    => \$print_config,
		'version|V'       => \$version,

		# afick options
		'archive=s'          => \$Archive,
		'database|D=s'       => \$Database,
		'ignore_case|a!'     => \$Ignore_case,
		'full_newdel|f!'     => \$Report_full_newdel,
		'history|y=s'        => \$History,
		'missing_files|m!'   => \$Warn_missing_file,
		'running_files|r!'   => \$Running,
		'dead_symlinks|s!'   => \$Warn_dead_symlinks,
		'exclude_suffix|x=s' => \$sufx_list,
		'timing|t!'          => \$Timing,
		'debug|d=i'          => \$Debug_level

	)
  )
{
	usage($Version);
	die "abort : incorrect option\n";
}

if ($help) {

	# -h : help
	usage($Version);
	exit;
}
elsif ($version) {

	# -V : version
	version($Version);
	exit;
}

if ($configfile) {
}
elsif ( -e $default_config_file ) {
	$configfile = $default_config_file;
}
else {
	usage($Version);
	die
"abort : missing configfile name (-c flag) and default config file $default_config_file\n";
}

# some more controls
if ( !-e $configfile ) {
	die "abort : missing configfile name $configfile\n";
}
elsif ( !-w $configfile ) {
	die "abort : configfile name $configfile is not writable\n";
}

if ($print_config) {
	print_config($configfile);
	exit;
}

my %changes;
$changes{'archive'}     = $Archive     if ( defined $Archive );
$changes{'database'}    = $Database    if ( defined $Database );
$changes{'debug'}       = $Debug_level if ( defined $Debug_level );
$changes{'history'}     = $History     if ( defined $History );
$changes{'ignore_case'} = $Ignore_case if ( defined $Ignore_case );
$changes{'report_full_newdel'} = $Report_full_newdel
  if ( defined $Report_full_newdel );
$changes{'running_files'} = $Running if ( defined $Running );
$changes{'timing'} = $Warn_dead_symlinks if ( defined $Warn_dead_symlinks );
$changes{'timing'} = $Warn_missing_file  if ( defined $Warn_missing_file );
$changes{'timing'} = $sufx_list          if ( defined $sufx_list );
$changes{'timing'} = $Timing             if ( defined $Timing );

my $return_value = change_config( $configfile, %changes );

exit $return_value;

__END__
